@@ -6,11 +6,13 @@ module Faraday
   class Response
     # Parse response bodies as JSON.
     class Json < Middleware
-      def initialize(app = nil, options = {})
+      def initialize(app = nil, parser_options: nil, content_type: /\bjson$/, preserve_raw: false)
         super(app)
-        @parser_options = options[:parser_options]
-        @content_types = Array(options[:content_type] || /\bjson$/)
-        @preserve_raw = options[:preserve_raw]
+        @parser_options = parser_options
+        @content_types = Array(content_type)
+        @preserve_raw = preserve_raw
+
+        process_parser_options
       end
 
       def on_complete(env)
@@ -27,7 +29,11 @@ module Faraday
       end
 
       def parse(body)
-        ::JSON.parse(body, @parser_options || {}) unless body.strip.empty?
+        return if body.strip.empty?
+
+        decoder, method_name = @decoder_options
+
+        decoder.public_send(method_name, body, @parser_options || {})
       end
 
       def parse_response?(env)
@@ -47,6 +53,19 @@ module Faraday
         type = type.split(';', 2).first if type.index(';')
         type
       end
+
+      def process_parser_options
+        @decoder_options = @parser_options&.delete(:decoder)
+
+        @decoder_options =
+          if @decoder_options.is_a?(Array) && @decoder_options.size >= 2
+            @decoder_options.slice(0, 2)
+          elsif @decoder_options.respond_to?(:load)
+            [@decoder_options, :load]
+          else
+            [::JSON, :parse]
+          end
+      end
     end
   end
 end
